package at.ac.tuwien.ims.acto_framework;

import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.util.Log;
import android.view.View;
import android.widget.TextView;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.graphics.Color;

/**
 * ACTOActivity
 * 	class to build GUIs for ACTO_framework.
 * 	provides control functions for the framework, communication structure and various helper functions
 * 
 * @author Emanuel Vonach
 */
public class ACTOActivity extends TRKMetaioActivity {
	
	private final String TAG = this.getClass().getSimpleName();	
	    
    // true if Tracker and TUIOServer are ready
	private boolean mTuioServerIsBound;
    
	// Messenger for communicating with the TUIOServer Service.
    private Messenger mOutMsgSvc_TuioServer = null;
    
    // Target we publish for clients to send messages to IncomingHandler.
    private final Messenger mIncomingMessenger = new Messenger(new IncomingHandler());
    

	//
	// LIFECYCLE
	/////////////////////
	
    /**
	 * onCreate: try to avoid this! Typical onCreate stuff has to be done in onTrackerReady(), because UI is handled by tracker!
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		
		super.onCreate(savedInstanceState);
		
		// if TUIOServer can be started (without rendering), check here if running and get settings
	}
	
	@Override
	protected void onNewIntent(Intent intent) {
		//someone started from notification, USB intent or from background
		//TODO: do something useful
	}
	
	//TODO: lifecycle should be implemented properly
//    @Override
//    protected void onStart() {
//        super.onStart();
//        // The activity is about to become visible.
//    }
//    @Override
//    protected void onResume() {
//        super.onResume();
//        // The activity has become visible (it is now "resumed").
//    }
//    @Override
//    protected void onPause() {
//        super.onPause();
//        // Another activity is taking focus (this activity is about to be "paused").
//    }
//    @Override
//    protected void onStop() {
//       // doUnbindService();
//        super.onStop();
//        // The activity is no longer visible (it is now "stopped")
//    }
	
    @Override
    protected void onDestroy() {
    	
        if (mTuioServerIsBound) {
        	doUnbindService();
        }
        super.onDestroy();
    }
	
	
	//
	// LIFECYCLE END
	/////////////////////

    
    
    

	
    //
	// TUIO COMMUNICATION
	/////////////////////
    
	public void testSth(int one, int two) {
		sendToServer(ACTOConst.SVC_TEST_STH, one, two);
	}
    
    /** request functions:
     * 		to request TUIO data from TUIOServer. Answer is always asynchronous via on...() function
     */
    
	/** requestTuioPose: ask for pose update of all TUIOs */			//reserved
	public void requestFullPose() {
		sendToServer(ACTOConst.SVC_ALL_POSE, 0, 0);
	}
//
//	public void requestFullExt() {
//		sendToServer(ACTOConst.SVC_FULL_EXT, 0, 0);
//	}
//
//	public void requestFullTrans() {
//		sendToServer(ACTOConst.SVC_FULL_TRANS, 0, 0);
//	}
//
//	public void requestFullRot() {
//		sendToServer(ACTOConst.SVC_FULL_ROT, 0, 0);
//	}
//
//    /** requestTuioState: ask for full state update of TUIO */		//reserved
//    public void requestTuioState(byte id) {
//    	sendToServer(ACTOConst.SVC_GET_STATE, id, 0);

    /** requestTuioPose: ask for pose update of TUIO */				//reserved
	public void requestTuioPose(byte id) {
		sendToServer(ACTOConst.SVC_GET_POSE, id, 0);
	}

    /** requestTuioExt: ask for extension update of TUIO */
	public void requestTuioExt(byte usbMsg, ACTOData adExt) {
		sendToServer(ACTOConst.SVC_GET_EXT, usbMsg, ACTOData.ADataToBundle(adExt));
	}

    /** requestTuioTrans: ask for translation update of TUIO */		//reserved
	public void requestTuioTrans(byte id) {
		sendToServer(ACTOConst.SVC_GET_TRANS, id, 0);
	}

    /** requestTuioRot: ask for rotation update of TUIO */			//reserved
	public void requestTuioRot(byte id) {
		sendToServer(ACTOConst.SVC_GET_ROT, id, 0);
	}
    
    /** requestPlaygroundSize: ask for current playground size */
	public void requestPlaygroundSize() {
		sendToServer(ACTOConst.SVC_GET_PLSIZE, 0, 0);
	}
	
    /** set functions:
     * 		to set TUIO data on TUIOServer. Answer is always asynchronous via on...() function
     */
	
//	/** setTuioState: set complete state of TUIO */				//reserved
//    public void setTuioState(ACTOData adState) {
//    	sendToServer(ACTOConst.SVC_SET_STATE, ACTOData.ADataToBundle(adState));
//    }

    /** setTuioPose: set pose of TUIO
     * @param  float[3] translation 
	 * @param  float[4] rotation */					//reserved
	public void setTuioPose(byte tuioId, float[] translation, float[] rotation) {
		ACTOData tmpPose = new ACTOData(ACTOData.POSE, tuioId, translation, rotation);
		sendToServer(ACTOConst.SVC_SET_POSE, ACTOData.ADataToBundle(tmpPose));
	}
	public void setTuioPose(ACTOData adPose) {
		sendToServer(ACTOConst.SVC_SET_POSE, ACTOData.ADataToBundle(adPose));
	}

	/** setTuioExt: set extension state of TUIO */	//TODO: separate functions for every possibility, or keep ACTOData??
	public void setTuioExt(byte usbMsg, ACTOData adExt) {
		sendToServer(ACTOConst.SVC_SET_EXT, usbMsg, ACTOData.ADataToBundle(adExt));
	}

	/** setTuioTrans: set position of TUIO */						//reserved
	public void setTuioTrans(byte tuioId, float[] translation) {
		ACTOData tmpTrans = new ACTOData(ACTOData.TRANS, tuioId, translation);
		sendToServer(ACTOConst.SVC_SET_TRANS, ACTOData.ADataToBundle(tmpTrans));
	}
	public void setTuioTrans(ACTOData adTrans) {
		sendToServer(ACTOConst.SVC_SET_TRANS, ACTOData.ADataToBundle(adTrans));
	}

	/** setTuioRot: set rotation of TUIO */							//reserved
	public void setTuioRot(byte tuioId, float[] rotation) {
		ACTOData tmpTrans = new ACTOData(ACTOData.TRANS, tuioId, rotation);
		sendToServer(ACTOConst.SVC_SET_ROT, ACTOData.ADataToBundle(tmpTrans));
	}
	public void setTuioRot(ACTOData adRot) {
		sendToServer(ACTOConst.SVC_SET_ROT, ACTOData.ADataToBundle(adRot));
	}
	
	/** setPlaygroundSize: set playground size */
	public void setPlaygroundSize(int[] playgroundSize) {
		if (playgroundSize.length == 3) {
			Bundle plSize = new Bundle();
			plSize.putByte(ACTOData.B_TYP, ACTOData.RES);
			plSize.putIntArray(ACTOData.B_RES, playgroundSize);
	
			sendToServer(ACTOConst.SVC_SET_PLSIZE, plSize);
		}
	}


    /** answers to set/request calls:
     * 		OVERRIDE these to get data!
     */
	
//    /** answer from server to set/requestTuioState(). */
//    protected void onTuioStateUpdate(ACTOData incomingState) {}

	/** answer from server to set/requestTuioPose() ALSO USED FOR BROADCAST.
	 * @param  float[3] translation 
	 * @param  float[4] rotation */
    protected void onTuioPoseUpdate(byte tuioId, float[] translation, float[] rotation) {}
    protected void onTuioPoseUpdate(ACTOData adPose) {}	

	/** answer from server to set/requestTuioExt(). */
    protected void onTuioExtUpdate(byte usbMsg, ACTOData incomingExt) {}

	/** answer from server to set/requestTuioTrans(). 
	 * @param  float[3] translation */
    protected void onTuioTransUpdate(byte tuioId, float[] translation) {}
    protected void onTuioTransUpdate(ACTOData adTrans) {}

	/** answer from server to set/requestTuioRot(). 
	 * @param  float[4] rotation */
    protected void onTuioRotUpdate(byte tuioId, float[] rotation) {}
    protected void onTuioRotUpdate(ACTOData adRot) {}
    
	/** answer from server to set/requestPlaygroundSize(). */
	protected void onPlaySizeUpdate(int[] playgroundSize) {}

    //
	// TUIO COMMUNICATION END
	/////////////////////



    
    
    //TODO: is this possible/practical?
//public void moveListener() {
//	//ID, T, R
//}
//public void extStateListener() {
//	// ID, EXTSTATE
//}

    
    /** 
     * requestServerStatus: ask if TUIOServer and Tracker are ready
     * @return mTuioServerIsBound: true if Tracker and TUIOServer are ready
    */
    public boolean requestServerStatus() {
    	//TODO: more useful with feedback if everything is ready to start app (including TUIOs initialized, Calibration etc)
    	return mTuioServerIsBound;
    }
	
	
    //
	// SIMPLE CON CONTROL FUNCTIONS
	/////////////////////
   
    /** request state functions:
     * 		to request connection states from TUIOServer. Answer is always asynchronous via on...() function
     */
	
    /** requestUsbState: ask for USB connection state from TUIOServer, answers via onUsbStateUpdate */
    public void requestUsbState() {
    	sendToServer(ACTOConst.SVC_GET_SETTING, TUIOServer.USB_CONNECTED, 0);
    }
    
    /** requestWifiBrcState:  get Broadcasting state from TUIOServer, answers via onAppBrcStateUpdate	 */
    public void requestAppBrcState() {
    	sendToServer(ACTOConst.SVC_GET_SETTING, TUIOServer.APP_BROADCASTING, 0);    	
    }
    
    /** requestWifiState:  get Wifi connection state from TUIOServer, answers via onWifiStateUpdate */
    public void requestWifiState() {
    	sendToServer(ACTOConst.SVC_GET_SETTING, TUIOServer.WIFI_CONNECTED, 0);
    }
    
    /** requestWifiBrcState:  get Broadcasting state from TUIOServer, answers via onWifiBrcStateUpdate	 */
    public void requestWifiBrcState() {
    	sendToServer(ACTOConst.SVC_GET_SETTING, TUIOServer.WIFI_BROADCASTING, 0);    	
    }
    
    /** requestCalibState:  get calibration state from TUIOServer, answers via onCalibStateUpdate	 */
    public void requestCalibState() {
    	sendToServer(ACTOConst.SVC_GET_SETTING, TUIOServer.CALIB_VALID, 0); 
    }
    
    /** set state functions:
     * 		to set communication states on TUIOServer. Answer is always asynchronous via on...() function
     */
    
    /** setAppBrcState: tell TUIOServer to set App Broadcasting state
 	 * @param int state: ACTOConst.STATE..., use STATE_UNDEFINED to toggle
 	 */
     protected void setAppBrcState(int state) {
     	sendToServer(ACTOConst.SVC_SET_SETTING, TUIOServer.APP_BROADCASTING, state);    	
     }
     
     /** setUsbState: tell TUIOServer to set USB connection state
 	 * @param int state: ACTOConst.STATE..., use STATE_UNDEFINED to toggle
 	 */
     protected void setUsbState(int state) {
     	sendToServer(ACTOConst.SVC_SET_SETTING, TUIOServer.USB_CONNECTED, state);
     }
     
     /** setWifiState: tell TUIOServer to set Wifi connection state
 	 * @param int statePort: ACTOConst.STATE..., use STATE_UNDEFINED to toggle, or set specific port
 	 */
     protected void setWifiState(int statePort) {
     	sendToServer(ACTOConst.SVC_SET_SETTING, TUIOServer.WIFI_CONNECTED, statePort);
     }
     
     /** setWifiBrcState: tell TUIOServer to set WIFI Broadcasting state
 	 * @param int state: ACTO_Const.STATE..., use STATE_UNDEFINED to toggle
 	 */
     protected void setWifiBrcState(int state) {
     	sendToServer(ACTOConst.SVC_SET_SETTING, TUIOServer.WIFI_BROADCASTING, state);    	
     }
     
     /** setCalibState: tell TUIOServer to set Calib state
 	 * @param int state: ACTOConst.STATE..., use STATE_UNDEFINED to toggle
 	 */
     protected void setCalibState(int state) {
     	//TODO: proper procedure
     	sendToServer(ACTOConst.SVC_SET_SETTING, TUIOServer.CALIB_VALID, state);    	
     }
    
    
    /** answers to set/request state calls:
     * 		OVERRIDE these to get data!
     */
    /** answer from server to set/requestUsbState(). */
    protected void onUsbStateUpdate(int state) {}
    
    /** answer from server to set/requestAppBrcState(). */
    protected void onAppBrcStateUpdate(int state) {}
    
    /** answer from server to set/requestWifiState(). */
    protected void onWifiStateUpdate(int state) {}
    
    //TODO: separate getInfo function?
    /** info on wifi connection as answer from server to set/requestWifiState(). */
    protected void onWifiInfoUpdate(String ip, int port) {}
    
    /** answer from server to set/requestWifiBrcState(). */
    protected void onWifiBrcStateUpdate(int state) {}
    
    /** answer from server to set/requestCalibState(). */
    protected void onCalibStateUpdate(int state) {}

    //
	// SIMPLE CON CONTROL FUNCTIONS END
	/////////////////////
    
	

    //
	// GUI ACCESS FUNCTIONS
    // all UI access has to be implemented likewise!
	/////////////////////
    
    /** setTextOnUI: UI Thread has to be accessed explicitly, because tracker/OpenGL manages UI  */
    protected void setTextOnUI(final TextView view, final String text) {
    	runOnUiThread(new  Runnable() { 
            public  void  run() { 
          	  view.setText(text);
            } 
      	});
    }
    
    /** setVisibilityOnUI: UI Thread has to be accessed explicitly, because tracker/OpenGL manages UI  */
    protected void setVisibilityOnUI(final View view, final int visibility) {
    	runOnUiThread(new  Runnable() { 
            public  void  run() { 
          	  view.setVisibility(visibility);
            } 
      	});
    }
    
    /** setColorOnUI: UI Thread has to be accessed explicitly, because tracker/OpenGL manages UI  */
    protected void setColorOnUI(final View view, final int color) {
    	runOnUiThread(new  Runnable() { 
            public  void  run() { 
          	  view.setBackgroundColor(color);
            } 
      	});
    }
    
    //
	// GUI ACCESS FUNCTIONS END
	/////////////////////
  
    
    
    //
    //LITTLE HELPERS
    /////////////////////////    
    
    /**
	 * getColorFromState: for colored state feedback
	 * @param int state: ACTO_Const.STATE...
	 * @return Color: returns color id corresponding to state
	 */
	protected int getColorFromState(int state) {
		switch (state) {
			case ACTOConst.STATE_UNDEFINED: return Color.GRAY;
			case ACTOConst.STATE_CLOSED: return Color.RED;
			case ACTOConst.STATE_PENDING: return Color.YELLOW;
			case ACTOConst.STATE_CONNECTED: return Color.GREEN;
		default:
			return Color.GRAY;
		}
	}
	
	//
    //LITTLE HELPERS END
    /////////////////////////    


    
    
    //
	// TRACKER Communication
    // functions are defined independently of tracking sdk
	/////////////////////
    
    /**
	 * getACTOLayout: You HAVE to override this, if you want to use your own layout (UI is handled by tracker!)
	 * @return R.layout.id: has to return the id of the desired layout
	 */
    
    @Override 
    protected int getACTOLayout() {
    	return R.layout.acto_activity;
    }
    
    /**
	 * onTrackerReady: event called after tracker is done initializing. GUI elements can now be accessed.
	 * 	if you override this, make sure to call super function! Otherwise TUIOServer will not be started
	 */
    @Override
    protected void onTrackerReady() {
    	   	  	
    	Log.d(TAG, "The Tracker is ready, starting Service");
		
    	// starting TUIOServer service
    	doBindService();
    }
    
    /**
	 * onFrameworkReady: event called after everything is done initializing. Use this instead of onCreate()!
	 */
    protected void onFrameworkReady() { }
    
    
    /**
	 * onNewTrackingData: called every frame by tracker, if new trackingdata is available. Sends raw data to TUIOServer for further processing.
	 * @param byte ID: COS ID of marker
	 * @param float[] translation: float[3] array with position data
	 * @param float[] quatRotation: float[4] array with rotation data
	 */
    @Override
    protected void onNewTrackingData(byte id, float[] translation, float[] quatRotation) {
		
    	if (mTuioServerIsBound) {
			Bundle trackingUpdate = new Bundle();
			trackingUpdate.putByte(ACTOData.B_TYP, ACTOData.POSE);
			trackingUpdate.putByte(ACTOData.B_ID, id);
			trackingUpdate.putFloatArray(ACTOData.B_TRANS, translation);
			trackingUpdate.putFloatArray(ACTOData.B_ROT, quatRotation);
	
	    	sendToServer(ACTOConst.SVC_TRK_UPDATE, trackingUpdate);
	    	
//	    	Log.d(TAG, "TT (AA): " + translation[0]);
	    	
//	    	Log.i(TAG, "RenderingFrameRate: " + metaioSDK.getRenderingFrameRate());
//	    	Log.i(TAG, "TrackingFrameRate: " + metaioSDK.getTrackingFrameRate());
//	    	Log.i(TAG, "CameraFrameRate: " + metaioSDK.getCameraFrameRate());
    	}
    }
    
    //
	// TRACKER Communication END
	/////////////////////

    

    //
    // SERVICE COMMUNICATION (INCOMING) from TUIOServer service
    ///////////////////////////
    /**
	 * IncomingHandler: handles incoming messages from TUIOServer
	 */
	class IncomingHandler extends Handler {
        
		// INCOMING messages are handled here!
		@Override
        public void handleMessage(Message incomingMsg) {

			switch (incomingMsg.what) {

				case ACTOConst.SVC_INIT_DONE:
					onFrameworkReady();					
					break;
			
//				case ACTOConst.SVC_SET_STATE:
//					onTuioStateUpdate(ACTOData.BundletoAData(incomingMsg.getData()));					
//					break;
				
				case ACTOConst.SVC_SET_POSE:
					ACTOData tmpPose = ACTOData.BundletoAData(incomingMsg.getData());
					onTuioPoseUpdate(tmpPose);
					onTuioPoseUpdate(tmpPose.id, tmpPose.translation, tmpPose.rotation);					
					break;
					
				case ACTOConst.SVC_SET_EXT:
					onTuioExtUpdate((byte)incomingMsg.arg1, ACTOData.BundletoAData(incomingMsg.getData()));					
					break;
					
				case ACTOConst.SVC_SET_TRANS:
					ACTOData tmpTrans = ACTOData.BundletoAData(incomingMsg.getData());
					onTuioTransUpdate(tmpTrans);
					onTuioTransUpdate(tmpTrans.id, tmpTrans.translation);					
					break;
					
				case ACTOConst.SVC_SET_ROT:
					ACTOData tmpRot = ACTOData.BundletoAData(incomingMsg.getData());
					onTuioRotUpdate(tmpRot);
					onTuioRotUpdate(tmpRot.id, tmpRot.rotation);		
					break;
					
				case ACTOConst.SVC_SET_PLSIZE:
					onPlaySizeUpdate(incomingMsg.getData().getIntArray(ACTOData.B_RES));					
					break;

					
				case ACTOConst.SVC_INFO_WIFI:
					//TODO: also as ACTOData.B_IP usw?
					Bundle wifiInfo = incomingMsg.getData();
					onWifiInfoUpdate(wifiInfo.getString("IP"), wifiInfo.getInt("port"));
					break;
	
//				case ACTOConst.SVC_INFO_USB:
//					break;

         		case ACTOConst.SVC_SET_SETTING:
         			switch(incomingMsg.arg1) {
         				case TUIOServer.USB_CONNECTED:
         					onUsbStateUpdate(incomingMsg.arg2);
         					break;
         				case TUIOServer.APP_BROADCASTING:
         					onAppBrcStateUpdate(incomingMsg.arg2);
         					break;
         				case TUIOServer.WIFI_CONNECTED:
         					onWifiStateUpdate(incomingMsg.arg2);
         					break;
         				case TUIOServer.WIFI_BROADCASTING:
         					onWifiBrcStateUpdate(incomingMsg.arg2);
         					break;
         				case TUIOServer.CALIB_VALID:
         					onCalibStateUpdate(incomingMsg.arg2);
         					break;
         				default:
         					Log.d(TAG, "UNKNOWN SETTING: " + incomingMsg.arg1 + ", state: " + incomingMsg.arg2);
         					//TODO: do something meaningful?, z.B. NAK message
         			}
         			break;

	            default:
	                super.handleMessage(incomingMsg);
            }
        }
    }

	
	
	
    //
	// SERVICE COMMUNICATION (OUTGOING) to TUIOServer
	////////////////  

	/**
	 * sendToServer: use this to send simple messages to TUIOServer
	 * @param int outgoingMsgCode: MSG code, defined in ACTO_Const
	 * @param int outgoingValue1: simple int value, depending on MSG
	 * @param int outgoingValue2: simple int value, depending on MSG
	 */
    private void sendToServer(int outgoingMsgCode, int outgoingValue1, int outgoingValue2) {
    	if (mTuioServerIsBound) {
			if (mOutMsgSvc_TuioServer != null) {
	            try {
	            	Message ougoingMsg = Message.obtain(null, outgoingMsgCode, outgoingValue1, outgoingValue2);
	            	ougoingMsg.replyTo = mIncomingMessenger;
	                mOutMsgSvc_TuioServer.send(ougoingMsg);
	            } catch (RemoteException e) {
	                // There is nothing special we need to do if the service
	                // has crashed.
	            }
			}
		}
    }
    
	/**
	 * sendToServer: use this for more complex messages to ACTO_Const
	 * @param int outgoingMsgCode: MSG code, defined in ACTO_Const
	 * @param int outgoingData: send any data as a Bundle, depending on MSG
	 */
    private void sendToServer(int outgoingMsgCode, Bundle outgoingData) {
    	if (mTuioServerIsBound) {
			if (mOutMsgSvc_TuioServer != null) {
	            try {
	            	Message ougoingMsg = Message.obtain(null, outgoingMsgCode, 0, 0);
	            	ougoingMsg.setData(outgoingData);
	            	ougoingMsg.replyTo = mIncomingMessenger;
	                mOutMsgSvc_TuioServer.send(ougoingMsg);
	            } catch (RemoteException e) {
	                // There is nothing special we need to do if the service
	                // has crashed.
	            }
			}
		}
    }
	
	/**
	 * sendToServer: use this for usb messages to TUIOs
	 * @param int outgoingMsgCode: MSG code, defined in ACTO_Const
	 * @param int usbMsgCode: MSG code, defined in ACTO_Const
	 * @param int outgoingData: send any data as a Bundle, depending on MSG
	 */
    private void sendToServer(int outgoingMsgCode, int usbMsgCode, Bundle outgoingData) {
    	if (mTuioServerIsBound) {
			if (mOutMsgSvc_TuioServer != null) {
	            try {
	            	Message ougoingMsg = Message.obtain(null, outgoingMsgCode, usbMsgCode, 0);
	            	ougoingMsg.setData(outgoingData);
	            	ougoingMsg.replyTo = mIncomingMessenger;
	                mOutMsgSvc_TuioServer.send(ougoingMsg);
	            } catch (RemoteException e) {
	                // There is nothing special we need to do if the service
	                // has crashed.
	            }
			}
		}
    }
	
    // Class for interacting with the main interface of the TUIOServer service.
    private ServiceConnection mConnection = new ServiceConnection() {
        public void onServiceConnected(ComponentName className, IBinder service) {
        	// object for interacting with the service.
        	mOutMsgSvc_TuioServer = new Messenger(service);
          
            //now we are bound
        	mTuioServerIsBound = true;
            
        	//setTextOnUI(logTextView, "Attached.");
            
            try {
            	// IMPORTANT: This also initializes the menu!
                Message msg = Message.obtain(null, ACTOConst.SVC_REG_CLIENT);
                msg.replyTo = mIncomingMessenger;
                mOutMsgSvc_TuioServer.send(msg);
                
            } catch (RemoteException e) {
                // In this case the service has crashed, there is no need to do anything here.
            }
            
            Log.d(TAG, "remote service TUIOServer connected"); 
        }

        public void onServiceDisconnected(ComponentName className) {
            // service crashed unexpectedly
        	//setTextOnUI(logTextView, "Disconnected.");
        	
        	mOutMsgSvc_TuioServer = null;
            
            Log.d(TAG, "service TUIOServer crashed unexpectedly");
            
            // now we aren't connected
            mTuioServerIsBound = false;
        }
    };
    
	private void doBindService() {
	    // Establish a connection with the service.
		bindService(new Intent(ACTOActivity.this, TUIOServer.class), mConnection, Context.BIND_AUTO_CREATE);
	
		// start here, if server could run without rendering in Activity
		
		//setTextOnUI(logTextView, "Binding.");
	}

	private void doUnbindService() {
	    if (mTuioServerIsBound) {
	        // If we are registered, unregister here
	        if (mOutMsgSvc_TuioServer != null) {
	            try {
	                Message msg = Message.obtain(null, ACTOConst.SVC_UNREG_CLIENT);
	                msg.replyTo = mIncomingMessenger;
	                mOutMsgSvc_TuioServer.send(msg);
	            } catch (RemoteException e) {
	                // the service has crashed
	            }
	        }
	        
	        // Detach our existing connection.
	        unbindService(mConnection);
	      
	        // if started, then also stop here
	        
	        //setTextOnUI(logTextView, "Unbinding");
	    }
	}          
    
    //
	// SERVICE COMMUNICATION END
	////////////////   
    

	
}
