package at.ac.tuwien.ims.acto_framework;

import android.R.bool;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.SeekBar.OnSeekBarChangeListener;

/**
 * ACTOGUI 
 * 	provides basic User Interface to control the ACTO_framework for standalone operation
 * 
 * @author Emanuel Vonach
 */
public class ACTOGUI extends ACTOActivity {
	
	private final String TAG = this.getClass().getSimpleName();	
	
	// basic variables:
	Button startUSBButton;
	Button startWLANButton;
	Button startWifiBrcButton;
	Button startCalibButton;
	Button testButton;
	Button demoButton;
	
	SeekBar steerBar;
	SeekBar speedBar;
	byte steerAngle = 0;
	byte speed = 0;
    TextView steerBarText;
    TextView speedBarText;
	
	// TextView for Output
    TextView infoTextView;
    TextView logTextView;
	
    TextView wifiIPTextView;
    TextView wifiPortTextView;
    
    //DEMO
    long lastTime = 0;
    long nowTime = 0;
    boolean collision = false;
    int demoMode = 0;

	//
	// LIFECYCLE
	/////////////////////
	

//	@Override
//	public void onCreate(Bundle savedInstanceState) {
//		
//		super.onCreate(savedInstanceState);
//	}
	
//	@Override
//	protected void onNewIntent(Intent intent) {
//		super.onNewIntent(intent);
//	}
	
//    @Override
//    protected void onStart() {
//        super.onStart();
//        // The activity is about to become visible.
//    }
//    @Override
//    protected void onResume() {
//        super.onResume();
//        // The activity has become visible (it is now "resumed").
//    }
//    @Override
//    protected void onPause() {
//        super.onPause();
//        // Another activity is taking focus (this activity is about to be "paused").
//    }
//    @Override
//    protected void onStop() {
//       // doUnbindService();
//        super.onStop();
//        // The activity is no longer visible (it is now "stopped")
//    }
	
//    @Override
//    protected void onDestroy() {
//        super.onDestroy();
//    }
	
	
	//
	// LIFECYCLE END
	/////////////////////

    /**
     * Catch Events from TUIO_Server
     */
    
    //if app-broadcasting is enabled
    @Override
    protected void onTuioPoseUpdate(ACTOData incomingPose) {
    	
    	//DEMO
    	if (demoMode == 1) {
	    	if (incomingPose.id == '1') {
	        	Log.d(TAG, "TUIOPoseUpdate-- id: " + incomingPose.id + ", position: " + incomingPose.translation[0] + ", " + incomingPose.translation[1] + ", " + incomingPose.translation[2]);

	    		nowTime = (long) System.currentTimeMillis();
	    		if (incomingPose.translation[0] > 800 && (nowTime - lastTime) > 4000 ) {
	    			if (incomingPose.translation[1] < 300) {
	    				lastTime = (long) System.currentTimeMillis();
	    				Log.d(TAG, "FAHRE NACH 200,400");
	    				setTuioTrans((byte)'1', new float[] {300, 450, 0});
	    			}
	    			else if (incomingPose.translation[1] > 450) {
	    				lastTime = (long) System.currentTimeMillis();
	    				Log.d(TAG, "KOLLIDIERE NACH 200,200");
	    				setTuioTrans((byte)'1', new float[] {300, 200, 0});
	    				setTuioTrans((byte)'2', new float[] {300, 500, 0});
	    				collision = true;
	    			}
	
	    		}
	    		else if (collision == true && (nowTime - lastTime) > 1000 && (nowTime - lastTime) < 2000) {
	    			setTuioPose((byte)'1', incomingPose.translation, incomingPose.rotation);
	    			Log.d(TAG, "WARTE KOLLISION");
	    		}
	    		else if (collision == true && (nowTime - lastTime) > 1500) {
	    			collision = false;
	    			setTuioTrans((byte)'1', new float[] {300, 200, 0});
	    			Log.d(TAG, "KOLLISION VORBEI 300 200");
	    		}
    		}
	    	
	    	if (incomingPose.id == '3') {

	    		nowTime = (long) System.currentTimeMillis();
	    		if (incomingPose.translation[0] > 1000 && (nowTime - lastTime) > 1000 && (nowTime - lastTime) < 4000) {
	    			lastTime = (long) System.currentTimeMillis();
	    			setTuioTrans((byte)'1', new float[] {300, incomingPose.translation[1], 0});
	    			setTuioTrans((byte)'2', new float[] {600, incomingPose.translation[1], 0});
	    			setTuioTrans((byte)'3', new float[] {900, incomingPose.translation[1], 0});

	    			setTuioExt(ACTOConst.USB_SET_DI_UNIT, new ACTOData(ACTOData.DATA_BYTE, (byte) '3', (byte) 1, (byte) 0));
	    			setTuioExt(ACTOConst.USB_SET_DI_UNIT, new ACTOData(ACTOData.DATA_BYTE, (byte) '3', (byte) 3, (byte) 1));
	    		}
	    		else if ((nowTime - lastTime) > 1000) {
	    			lastTime = (long) System.currentTimeMillis();
	    			setTuioExt(ACTOConst.USB_SET_DI_UNIT, new ACTOData(ACTOData.DATA_BYTE, (byte) '3', (byte) 1, (byte) 1));
	    			setTuioExt(ACTOConst.USB_SET_DI_UNIT, new ACTOData(ACTOData.DATA_BYTE, (byte) '3', (byte) 3, (byte) 0));
	    		}
	    	}
    	}
    	
    	


    	//(long) (System.nanoTime());
    	
    	//Log.d(TAG, "TUIOPoseUpdate-- id: " + incomingPose.id + ", position: " + incomingPose.translation[0] + ", " + incomingPose.translation[1] + ", " + incomingPose.translation[2]);
    }	

    
	@Override
    protected void onUsbStateUpdate(int state) {
    	setColorOnUI(startUSBButton, getColorFromState(state));
    }
    
	@Override
    protected void onAppBrcStateUpdate(int state) {
    	if (state == ACTOConst.STATE_PENDING || state == ACTOConst.STATE_CONNECTED)	{
    		setTextOnUI(infoTextView, "app broadcasting activated");
    		Log.d(TAG, "app broadcasting activated");
		}
    	else {
    		setTextOnUI(infoTextView, "app broadcasting deactivated");
    		Log.d(TAG, "app broadcasting deactivated");
    	}
    }
    
	@Override
    protected void onWifiStateUpdate(int state) {
    	setColorOnUI(startWLANButton, getColorFromState(state));
    	
		if (state != ACTOConst.STATE_CLOSED) {
			requestWifiBrcState();
			setVisibilityOnUI(startWifiBrcButton, View.VISIBLE);
			setVisibilityOnUI(wifiIPTextView, View.VISIBLE);
			setVisibilityOnUI(wifiPortTextView, View.VISIBLE);
		}
		else {
			setVisibilityOnUI(startWifiBrcButton, View.INVISIBLE);
			setVisibilityOnUI(wifiIPTextView, View.INVISIBLE);
			setVisibilityOnUI(wifiPortTextView, View.INVISIBLE);
		}
    }
    
	@Override
    protected void onWifiInfoUpdate(String ip, int port) {
		setTextOnUI(wifiIPTextView, getString(R.string.wifiIP) + ip);
		setTextOnUI(wifiPortTextView, getString(R.string.wifiPort) + port);
	}
	
	@Override
    protected void onWifiBrcStateUpdate(int state) {
    	setColorOnUI(startWifiBrcButton, getColorFromState(state));
    }
    
	@Override
    protected void onCalibStateUpdate(int state) {
    	setColorOnUI(startCalibButton, getColorFromState(state));
    }

    
    /**
	 * getACTOLayout: You HAVE to override this, if you want to use your own layout (UI is handled by tracker!)
	 * @return R.layout.id: has to return the id of the desired layout
	 */
    @Override 
    protected int getACTOLayout() {
    	return R.layout.acto_activity;
    }
    
    
    /**
	 * onFrameworkReady: event called after everything is done initializing. Use this instead of onCreate()!
	 */
    protected void onFrameworkReady() {
    	
    	//get GUI Views
    	infoTextView = (TextView)findViewById(R.id.info_text);
    	logTextView = (TextView)findViewById(R.id.log_text);
        wifiIPTextView = (TextView)findViewById(R.id.wifiip_text);
        wifiPortTextView = (TextView)findViewById(R.id.wifiport_text);
    	
    	startUSBButton = (Button)findViewById(R.id.startUSBButton);
		startWLANButton = (Button)findViewById(R.id.startWLANButton);
		startWifiBrcButton = (Button)findViewById(R.id.startWifiBrcButton);
		startCalibButton = (Button)findViewById(R.id.startCalibButton);
		
		testButton = (Button)findViewById(R.id.testButton);
		demoButton = (Button)findViewById(R.id.demoButton);
		steerBar = (SeekBar)findViewById(R.id.steerBar);
		speedBar = (SeekBar)findViewById(R.id.speedBar);
		steerBarText = (TextView)findViewById(R.id.steerBarText);
	    speedBarText = (TextView)findViewById(R.id.speedBarText);
    	
		enableSeekBarListener();
    	//show menu
		changeMenuVisibility(View.VISIBLE);

    	
    	setTextOnUI(logTextView, "menu initialized");
    	setTextOnUI(logTextView, "server running");
    	setTextOnUI(infoTextView, "server running");
    }

	
	//
	// BUTTONS
    ////////////////

    public void startUSBButtonListener(View v) {
    	setUsbState(ACTOConst.STATE_UNDEFINED);
	}

    public void startWLANButtonListener(View v) {
    	//TODO: probably implement port properly with input textview 
    	setWifiState(ACTOConst.STATE_UNDEFINED);
	}
    
    public void startWifiBrcButtonListener(View v) {
    	setWifiBrcState(ACTOConst.STATE_UNDEFINED);    	
	}
    
    public void startCalibButtonListener(View v) {
		setCalibState(ACTOConst.STATE_UNDEFINED);
    }
    
//    boolean test_tog = false;
//    byte test = 0;
    
    public void testButtonListener(View v) {
//    	Log.i(TAG, "angle:" + steerAngle + " speed:" + speed);
//    	setTuioExt(ACTOConst.USB_MOTOR, new ACTOData(ACTOData.DATA_BYTE, (byte)'1', steerAngle, speed));
    	
//    	setTuioExt(ACTOConst.USB_PING, new ACTOData(ACTOData.ID, (byte)'1'));
//    	Log.i(TAG, "PING: " + '1');
    	
    	//setTuioExt(ACTOConst.USB_MOTOR_MS, new ACTOData(ACTOData.MOTOR_MS, (byte)'1', steerAngle, speed, 1000));
    	
    	
//    	if (test > 2) test = 0;
//    	
//    	if (!test_tog) {
//    	setTuioExt(ACTOConst.USB_SET_DI_UNIT, new ACTOData(ACTOData.DATA_BYTE, (byte)'1', test, (byte)1));
//    	test_tog = true;
//    	}
//    	else {
//    		setTuioExt(ACTOConst.USB_SET_DI_UNIT, new ACTOData(ACTOData.DATA_BYTE, (byte)'1', test, (byte)0));
//    		test_tog = false;
//    		test++;
//    	}
    	

    	//DEMO
    	setAppBrcState(ACTOConst.STATE_CONNECTED);
    	demoMode = 2;
    	lastTime =  (long) (System.currentTimeMillis());
    	nowTime =  (long) (System.currentTimeMillis());
    	
 //   	    	testSth(0,0);
//    	Log.i(TAG, "TT START ! ");
    	
    	
    	
//    	Log.i(TAG, "RenderingFrameRate: " + metaioSDK.getRenderingFrameRate());
//    	Log.i(TAG, "TrackingFrameRate: " + metaioSDK.getTrackingFrameRate());
//    	Log.i(TAG, "CameraFrameRate: " + metaioSDK.getCameraFrameRate());

    }
    
    public void demoButtonListener(View v) {
    	//DEMO
    	setAppBrcState(ACTOConst.STATE_CONNECTED);
    	demoMode = 1;
    	lastTime =  (long) (System.currentTimeMillis());
    	nowTime =  (long) (System.currentTimeMillis());
    }
    
    public void closeActivityButtonListener(View v) {
    	finish();
    }
        
	private void enableSeekBarListener() {
		try {
		    //Listener to receive changes to the SeekBar1's progress level
			steerBar.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
			    public void onStopTrackingTouch(SeekBar arg0) {
			    	//NOT MEANT TO BE USED DIRECTLY! Always use setTuioPose/Trans/Rot ! This is just for testing!
			    	setTuioExt(ACTOConst.USB_MOTOR_MS, new ACTOData(ACTOData.MOTOR_MS, (byte)'1', steerAngle, speed, 1000));
			    	setTuioExt(ACTOConst.USB_MOTOR_MS, new ACTOData(ACTOData.MOTOR_MS, (byte)'2', steerAngle, speed, 1000));
			    }
			 
			    public void onStartTrackingTouch(SeekBar arg0) {
			    }
			    //When progress level of seekbar1 is changed
			    public void onProgressChanged(SeekBar arg0, int progress, boolean arg2) {
			    	int temp = progress-128;
			    	if (temp < 0) temp = temp + 256;
			    	steerAngle = (byte) temp;
			    }
			});
			
			//Listener to receive changes to the SeekBar2's progress level
			speedBar.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
				public void onStopTrackingTouch(SeekBar arg0) {
					setTuioExt(ACTOConst.USB_MOTOR_MS, new ACTOData(ACTOData.MOTOR_MS, (byte)'1', steerAngle, speed, 1000));
					setTuioExt(ACTOConst.USB_MOTOR_MS, new ACTOData(ACTOData.MOTOR_MS, (byte)'2', steerAngle, speed, 1000));
				}
		 
				public void onStartTrackingTouch(SeekBar arg0) {
				}
				//When progress level of seekbar2 is changed
				public void onProgressChanged(SeekBar arg0, int progress, boolean arg2) {
			    	speed = (byte)progress;
				}
			});
		 
		} catch (Exception e) {
		    e.printStackTrace();
		}
	}

	private void changeMenuVisibility(int visibility) { 

		setVisibilityOnUI(startUSBButton, visibility);
		setVisibilityOnUI(startWLANButton, visibility);
		//if (wifiConnected > 0) setVisibilityOnUI(startWifiBrcButton, visibility);
		setVisibilityOnUI(startCalibButton, visibility);
		
		setVisibilityOnUI(testButton, visibility);
		setVisibilityOnUI(demoButton, visibility);
		setVisibilityOnUI(steerBar, visibility);
		setVisibilityOnUI(speedBar, visibility);
		setVisibilityOnUI(steerBarText, visibility);
		setVisibilityOnUI(speedBarText, visibility);

	}


}
