package at.ac.tuwien.ims.acto_framework;

//import android.content.Intent;
//import android.hardware.Camera;
import android.hardware.Camera.CameraInfo;
import android.os.Bundle;
import android.util.Log;


import com.metaio.sdk.ARViewActivity;
import com.metaio.sdk.jni.IGeometry;
import com.metaio.sdk.jni.IMetaioSDKCallback;

//from template
import com.metaio.sdk.MetaioDebug;
import com.metaio.sdk.jni.ETRACKING_STATE;
//import com.metaio.sdk.jni.EVISUAL_SEARCH_STATE;
//import com.metaio.sdk.jni.IMetaioSDKAndroid;
//import com.metaio.sdk.jni.IVisualSearchCallback;
//import com.metaio.sdk.jni.ImageStruct;
//import com.metaio.sdk.jni.Rotation;
//import com.metaio.sdk.jni.TrackingValues;
import com.metaio.sdk.jni.TrackingValuesVector;
import com.metaio.sdk.jni.Vector2d;
import com.metaio.sdk.jni.Vector3d;
import com.metaio.sdk.jni.Vector4d;
//import com.metaio.sdk.jni.VisualSearchResponseVector;
import com.metaio.tools.SystemInfo;
import com.metaio.tools.io.AssetsManager;

/**
 * TRK_MetaioActivity
 * 	Interface to metaio sdk for ACTO framework. 
 * 
 * @author Emanuel Vonach
 */
public abstract class TRKMetaioActivity extends ARViewActivity {

	private final String TAG = this.getClass().getSimpleName();
	
	private IGeometry mModel1;
	private IGeometry mModel2;
	private IGeometry mModel3;
	private IGeometry mModel4;
	private MetaioSDKCallbackHandler mCallbackHandler;
	
	//
	// LIFECYCLE BEGIN
	/////////////////////
	
	@Override
	public void onCreate(Bundle savedInstanceState) 
	{
		super.onCreate(savedInstanceState);
		
		mCallbackHandler = new MetaioSDKCallbackHandler();
		
		metaioSDK.setTrackingEventCallbackReceivesAllChanges(true);
		
//		// Enable metaio SDK debug log messages based on build configuration
//		MetaioDebug.enableLogging(BuildConfig.DEBUG);
	}
	
	//
	// LIFECYCLE END
	/////////////////////
	
	
	//
	// METAIO Overrides
	/////////////////////
	
	@Override
	protected int getGUILayout() {
		// Attaching layout to the activity
		return getACTOLayout();
	}

	@Override
	protected IMetaioSDKCallback getMetaioSDKCallbackHandler() 	
	{
		return mCallbackHandler;
	}

	@Override
	protected void onGeometryTouched(IGeometry geometry) {
		// TODO: Auto-generated method stub
	}
	
	@Override
	public void onDrawFrame() 
	{
		super.onDrawFrame();
		//handleTrackingData(metaioSDK.getTrackingValues());
	}
	
	@Override
	protected void startCamera()
	{
		final int cameraIndex = SystemInfo.getCameraIndex(CameraInfo.CAMERA_FACING_FRONT);
//		final int cameraIndex = SystemInfo.getCameraIndex(CameraInfo.CAMERA_FACING_BACK);
		
		//TODO: test different res, and downsample
		//metaioSDK.startCamera(cameraIndex, 320, 240, 2);
		metaioSDK.startCamera(cameraIndex, 320, 240);
		
		//TODO:Set Display Rotation right, if necessary, most likely after start, after init
//		Camera metaioCam = metaioSDK.getCamera(this);
//		metaioCam.setDisplayOrientation(0);
	}	
	
	//
	// METAIO Overrides END
	/////////////////////	
	
	
	
	
	
	//TODO: do something useable
	@Override
	protected void loadContents() 
	{
		try
		{
			//TODO: Extracting Assets should be done async somewhere
			AssetsManager.extractAllAssets(this, true);	
			
			// Getting a file path for tracking configuration XML file
			String trackingConfigFile = AssetsManager.getAssetPath(this.getApplicationContext(), "TrackingData_Marker.xml");
			//String trackingConfigFile = AssetsManager.getAssetPath(this.getApplicationContext(), "TrackingData_PictureMarker.xml");
			
			// Assigning tracking configuration
			boolean result = metaioSDK.setTrackingConfiguration(trackingConfigFile); 

			Log.d(TAG, "Tracking data loaded: " + result); 
			
			// Getting a file path for a 3D geometry
			String markerModel1 = AssetsManager.getAssetPath(this.getApplicationContext(), "marker1_img.png");
			String markerModel2 = AssetsManager.getAssetPath(this.getApplicationContext(), "marker2_img.png");
			String markerModel3 = AssetsManager.getAssetPath(this.getApplicationContext(), "marker3_img.png");
			String markerModel4 = AssetsManager.getAssetPath(this.getApplicationContext(), "marker4_img.png");
			
			loadMarkerModel(markerModel1, mModel1, 1);
			loadMarkerModel(markerModel2, mModel1, 2);
			loadMarkerModel(markerModel3, mModel1, 3);
			loadMarkerModel(markerModel4, mModel1, 4);
			
//			if (markerModel1 != null) 
//			{
//				// Loading 3D geometry
//				mModel1 = metaioSDK.createGeometryFromImage(markerModel1, false, false);
//				//mModel = metaioSDK.createGeometry(metaioManModel);
//				if (mModel1 != null) 
//				{
//					// Set geometry properties
//					mModel1.setScale(new Vector3d(0.5f, 0.5f, 0.5f));
//					
//					//set corresponding Marker
//					mModel1.setCoordinateSystemID(1);
//					
//				}
//				else
//					MetaioDebug.log(Log.ERROR, "Error loading geometry: "+markerModel1);
//			}
//			if (markerModel2 != null) 
//			{
//				// Loading 3D geometry
//				mModel2 = metaioSDK.createGeometryFromImage(markerModel2, false, false);
//				//mModel = metaioSDK.createGeometry(metaioManModel);
//				if (mModel2 != null) 
//				{
//					// Set geometry properties
//					mModel2.setScale(new Vector3d(0.5f, 0.5f, 0.5f));
//					
//					//set corresponding Marker
//					mModel2.setCoordinateSystemID(2);
//					
//				}
//				else
//					MetaioDebug.log(Log.ERROR, "Error loading geometry: "+markerModel2);
//			}
			
		}       
		catch (Exception e)
		{
			
		}
	}	
	
	//helper to load models and geometries on markers
	private void loadMarkerModel(String markerModel, IGeometry mModel, int cosId) {
		if (markerModel != null) 
		{
			// Loading 3D geometry
			mModel = metaioSDK.createGeometryFromImage(markerModel, false, false);
			//mModel = metaioSDK.createGeometry(metaioManModel);
			if (mModel != null) 
			{
				// Set geometry properties
				mModel.setScale(new Vector3d(0.5f, 0.5f, 0.5f));
				
				//set corresponding Marker
				mModel.setCoordinateSystemID(cosId);
			}
			else
				MetaioDebug.log(Log.ERROR, "Error loading geometry: "+markerModel);
		}
	}
	
	
	//
	// ABSTRACT METHODS for ACTOActivity access
	/////////////////////
	
	// get current content view
	protected abstract int getACTOLayout();
	
	
	
//	// TODO: make method to be called from ACTOActivity
//	public void getTrackingData(int ID) {
//		TrackingValues currentPose = metaioSDK.getTrackingValues(ID);
//		Log.d("METAIOACTIVITY", "quality: " + currentPose.getQuality());
//	}
	
    protected abstract void onNewTrackingData(byte id, float[] translation, float[] quatRotation);
    
    protected abstract void onTrackerReady();
    
    
    
	//
	// ABSTRACT METHODS for ACTOActivity END
	/////////////////////
  
	private void handleTrackingData (TrackingValuesVector trackingValues) {
		
		// how many markers are tracked right now?
		int uptSize = (int) trackingValues.size();
		//Log.d(TAG, "SIZE = " + uptSize);
		
		Vector3d trans;
		Vector4d quatRot;
		
		// for every tracked marker, give update to ACTOActivity
		for(int i=0; i<uptSize; i++) {

			trans = trackingValues.get(i).getTranslation();
			quatRot = trackingValues.get(i).getRotation().getQuaternion();
					
			//trackingValues.get(i).isTrackingState()
			if (trackingValues.get(i).getState() == ETRACKING_STATE.ETS_TRACKING) {
				// ID, float[3], float[4]
				onNewTrackingData((byte)trackingValues.get(i).getCoordinateSystemID(), new float[]{trans.getX(),trans.getY(),trans.getZ()}, new float[]{quatRot.getW(),quatRot.getX(),quatRot.getY(),quatRot.getZ()});
			
				//TODO: ACHTUNG AUSGABE IN SCREEN COORDINATES!!!
	//			Vector2d screen = metaioSDK.getScreenCoordinatesFrom3DPosition(trackingValues.get(i).getCoordinateSystemID(), trans);
	//    		onNewTrackingData((byte)trackingValues.get(i).getCoordinateSystemID(), new float[]{screen.getX(),screen.getY(),trans.getZ()}, new float[]{quatRot.getW(),quatRot.getX(),quatRot.getY(),quatRot.getZ()});
	    		//leaking float[]
			}
		}
	}
	
	
	
	//
	// METAIO Callback
	/////////////////////
	
	final class MetaioSDKCallbackHandler extends IMetaioSDKCallback 
	{

		@Override
		public void onSDKReady() 
		{
			onTrackerReady();
		}
		
		//called whenever tracking state changes (e.g. lost/found)
		@Override
		public void onTrackingEvent(TrackingValuesVector trackingValues)
		{
			handleTrackingData(trackingValues);
			//Log.d(TAG, "TRACKING EVENT!! quality: " + trackingValues.get(0).getQuality());
			//setTextOnUI((TextView)findViewById(R.id.test_text), "The tracking time is:" + trackingValues.get(1).getQuality());
			
			//COMMENT FROM METAIO FORUM: To track an ID Marker use TrackingValues.isTrackingState(ETRACKING_STATE.ETS_TRACKING) because the app occasianally receives a TrackingValue with status ETS_FOUND although the Marker ID was never tracked by the camera.
		}
		
//		@Override
//		public void onAnimationEnd(IGeometry geometry, String animationName) 
//		{
//			MetaioDebug.log("animation ended" + animationName);
//		}
//		
//		@Override
//		public void onMovieEnd(IGeometry geometry, String name)
//		{
//			MetaioDebug.log("movie ended" + name);
//		}
//		
//		@Override
//		public void onNewCameraFrame(ImageStruct cameraFrame)
//		{
//			//MetaioDebug.log("a new camera frame image is delivered" + cameraFrame.getTimestamp());
//		}
//		
//		@Override 
//		public void onCameraImageSaved(String filepath)
//		{
//			MetaioDebug.log("a new camera frame image is saved to" + filepath);
//		}
//		
//		@Override
//		public void onScreenshotImage(ImageStruct image)
//		{
//			MetaioDebug.log("screenshot image is received" + image.getTimestamp());
//		}
//		
//		@Override
//		public void onScreenshotSaved(String filepath)
//		{
//			MetaioDebug.log("screenshot image is saved to" + filepath);
//		}
//
//		@Override
//		public void onInstantTrackingEvent(boolean success, String file)
//		{
//			if (success)
//			{
//				MetaioDebug.log("Instant 3D tracking is successful");
//			}
//		}
	}
	
	//
	// METAIO Callback END
	/////////////////////
}
