package at.ac.tuwien.ims.acto_framework;

/**
 * TUIO_CommHub:
 * 		handles Wifi & Usb communication
 * 
 * @author Emanuel Vonach
 * 
 * based on code by Kaltenriner Christoph, Leichtfried Michael
 */

// TODO: Cleanup

import android.util.Log;
import java.io.Closeable;
import java.io.IOException;
import android.hardware.usb.UsbManager;





public class TUIOCommHub implements Closeable {
	
	private final String TAG = this.getClass().getSimpleName();
	
	private WifiTransmitter wifiTransmitter = null;
	private Thread wifiThread;
	
	private UsbSerialComm usbSerialComm = null;
	private Thread usbThread;
	
	private TUIOServer tuioServer = null;
	
	//default port
	private int wifiPort = 12435;
	//private String wifiIP = "";

	//constructor
    public TUIOCommHub(TUIOServer callingServer)
	{
    	tuioServer = callingServer;
	}
    
	@Override
	public void close() throws IOException {

		closeWifiConnection();
		closeUsbConnection();
		//TODO: kommt closeEvent noch an (fr StateManagement), sonst muss hier noch onWifiClosed ausgefhrt werden!
		Log.d(TAG, "CommHub cleaned up");
	}
	


//    // TODO: if adhoc, get ip from here!
//    public String getIP() {
//      return wifiIP;
//    }
    
    public int getPort() {
    	return wifiPort;
    }

    
    public void startWifi(int port) {
    	if (port != ACTOConst.STATE_UNDEFINED) wifiPort = port;
    	establishWifiConnection(wifiPort);
    }
    
    public void stopWifi() {
    	closeWifiConnection();
    }
    
    //Feedback from WifiTransmitter
    public void onWifiStateChange(int state) {
    	tuioServer.onWifiStateChange(state);
    }
    
    
    //TODO: implement
    public void startUsb(UsbManager usbManager) {
    	establishUsbConnection(usbManager);
    }
    
    public void stopUsb() {
    	closeUsbConnection();
    }

    //Feedback from USBSerial
    public void onUsbStateChange(int state) {
    	tuioServer.onUsbStateChange(state);
    }
    

    //TODO: use own functions to send answer!! NOT directly in wifiTransmitter.sendByte
    
    //
    // COMMUNICATION INBOUND FROM WIFI HANDLING
    //////////////////
	public void handleWifiMsg(byte wi_msg, ACTOData data){	//TODO: could also return boolean, for ACK!!!
		//Log.d(TAG, "Received Data: " + wi_msg);

		switch (wi_msg){
				
		//TODO: implement when data structure ready OR REMOVE!!!
			// simple CMD messages
//			case (ACTOConst.WI_FULL_STATE): case (ACTOConst.WI_FULL_POSE): case (ACTOConst.WI_FULL_EXT):
//			case (ACTOConst.WI_FULL_TRANS): case (ACTOConst.WI_FULL_ROT):
//				break;
			
			case (ACTOConst.WI_ACK):
				//wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				//do I want to know that here?
				break;
			
			//get state messages: CMD
			case (ACTOConst.WI_GET_ST_USB):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				wifiTransmitter.sendState(ACTOConst.WI_SET_ST_USB, tuioServer.getState(TUIOServer.USB_CONNECTED));
				break;
				
			case (ACTOConst.WI_GET_ST_APBRC): 
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				wifiTransmitter.sendState(ACTOConst.WI_SET_ST_APBRC, tuioServer.getState(TUIOServer.APP_BROADCASTING));
				break;
				
			case (ACTOConst.WI_GET_ST_WIFI):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				wifiTransmitter.sendState(ACTOConst.WI_SET_ST_WIFI, tuioServer.getState(TUIOServer.WIFI_CONNECTED));
				break;
			
			case (ACTOConst.WI_GET_ST_WIBRC):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				wifiTransmitter.sendState(ACTOConst.WI_SET_ST_WIBRC, tuioServer.getState(TUIOServer.WIFI_BROADCASTING));
				break;
			
			case (ACTOConst.WI_GET_ST_CAL):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				wifiTransmitter.sendState(ACTOConst.WI_SET_ST_CAL, tuioServer.getState(TUIOServer.CALIB_VALID));
				break;

			//set state messages: CMD, int, NOTE: double use of state: port or state
			case (ACTOConst.WI_SET_ST_USB):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				tuioServer.setUsbState(data.state);
				break;
			case (ACTOConst.WI_SET_ST_APBRC):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				tuioServer.setAppBrcState(data.state);
				break;
			case (ACTOConst.WI_SET_ST_WIFI):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				tuioServer.setWifiState(data.state);
				break;
			case (ACTOConst.WI_SET_ST_WIBRC):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				tuioServer.setWifiBrcState(data.state);
				break;
			case (ACTOConst.WI_SET_ST_CAL):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				tuioServer.setCalibState(data.state);
				break;

			case (ACTOConst.WI_ALL_POSE):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				tuioServer.getAllPoseUpdate();
				break;
				
			// read CMD + ID messages
//			case (ACTOConst.WI_GET_STATE):
//				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
//				wifiSendTuioState(tuioServer.getTuioState(data.id));
//				break;
			case (ACTOConst.WI_GET_POSE):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				wifiSendTuioPose(tuioServer.getTuioPose(data.id));
				break;
			//SEPERATE BECAUSE OF usb_msg
//			case (ACTOConst.WI_GET_EXT):
//				wifiSendTuioExt(tuioServer.getTuioExt(data.id));	
//				break;
			case (ACTOConst.WI_GET_TRANS):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				wifiSendTuioTrans(tuioServer.getTuioTrans(data.id));
				break;	
			case (ACTOConst.WI_GET_ROT):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				wifiSendTuioRot(tuioServer.getTuioRot(data.id));
				break;	
			
			case (ACTOConst.WI_GET_PLSIZE):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				wifiSendRes(tuioServer.getPlaygroundSize());//TODO: as ACTOData?
				break;
				
//			case (ACTOConst.WI_SET_STATE):
//				tuioServer.setTuioState(data);
//				break;

			case (ACTOConst.WI_SET_POSE):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				tuioServer.setTuioPose(data);
				break;
				//SEPERATE BECAUSE OF usb_msg
//			case (ACTOConst.WI_SET_EXT):	
//				tuioServer.setTuioExt(data);	
//				break;

			case (ACTOConst.WI_SET_TRANS):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				tuioServer.setTuioTrans(data);
				break;

			case (ACTOConst.WI_SET_ROT):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				tuioServer.setTuioRot(data);
				break;
			
			// CMD + int[] messages
			case (ACTOConst.WI_SET_PLSIZE):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				tuioServer.setPlaygroundSize(data.resolution);
				break;

			// message unknown					
			default:
				wifiTransmitter.sendByte(ACTOConst.WI_NAK);
				Log.e(TAG, "MESSAGE UNKNOWN: " + wi_msg);
				break;
		}	
			
	}
	
    //
    // COMMUNICATION INBOUND FROM WIFI WITH TARGET USB
    //////////////////
	public boolean handleWifiMsg(byte wi_msg, byte usb_msg, ACTOData data){
		//Log.d(TAG, "Received Data: " + wi_msg);

		switch (wi_msg){
			case (ACTOConst.WI_GET_EXT):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				return tuioServer.getTuioExt(usb_msg, data);
			
			case (ACTOConst.WI_SET_EXT):
				wifiTransmitter.sendByte(ACTOConst.WI_ACK);
				return tuioServer.setTuioExt(usb_msg, data);
			
			// message unknown					
			default:
				wifiTransmitter.sendByte(ACTOConst.WI_NAK);
				Log.e(TAG, "MESSAGE UNKNOWN: " + wi_msg);
		}	
		return false;
	}
	
    //
    // COMMUNICATION OUTBOUND OVER WIFI
    //////////////////
	protected boolean wifiSendByte(byte bt){
    	if(wifiTransmitter != null)
		{
			return wifiTransmitter.sendByte(bt);
		}
    	return false;
	}

    protected boolean wifiSendState(int setting ,int state){
    	if(wifiTransmitter != null)
		{
    		switch (setting) {
    			case TUIOServer.USB_CONNECTED:
    				return wifiTransmitter.sendState(ACTOConst.WI_SET_ST_USB, state);
    			case TUIOServer.APP_BROADCASTING:
    				return wifiTransmitter.sendState(ACTOConst.WI_SET_ST_APBRC, state);
    			case TUIOServer.WIFI_CONNECTED:
    				return wifiTransmitter.sendState(ACTOConst.WI_SET_ST_WIFI, state);
    			case TUIOServer.WIFI_BROADCASTING:
    				return wifiTransmitter.sendState(ACTOConst.WI_SET_ST_WIBRC, state);
    			case TUIOServer.CALIB_VALID:
    				return wifiTransmitter.sendState(ACTOConst.WI_SET_ST_CAL, state);
				default: return false;
    		}
			
		}
    	return false;
	}
    
    protected boolean wifiSendRes(int[] res){
    	if(wifiTransmitter != null)
		{
			return wifiTransmitter.sendRes(ACTOConst.WI_SET_PLSIZE, res);
		}
    	return false;
	}
	
    //vermutlich nicht ntig, mglich?, Pose und Ext getrennt
	protected boolean wifiSendTuioState(ACTOData cd) {
	//TODO: check for ACTOData.type necessary?
//		if(wifiTransmitter != null)
//		{
//			return wifiTransmitter.sendADIdState(ACTOConst.WI_SET_STATE, cd);
//		}
    	return false;
	}

	protected boolean wifiSendTuioTrans(ACTOData cd) {
    	//TODO: check for ACTOData.type necessary?
		if(wifiTransmitter != null)
		{
			return wifiTransmitter.sendADIdTrans(ACTOConst.WI_SET_TRANS, cd);
		}
		return false;
	}

    protected boolean wifiSendTuioRot(ACTOData cd) {
    	//TODO: check for ACTOData.type necessary?
		if(wifiTransmitter != null)
		{
			return wifiTransmitter.sendADIdRot(ACTOConst.WI_SET_ROT, cd);
		}
		return false;
	}
    
    protected boolean wifiSendTuioPose(ACTOData trackingUpdate) {
    	//TODO: check for ACTOData.type necessary?
    	if(wifiTransmitter != null)
		{
			return wifiTransmitter.sendADIdPose(ACTOConst.WI_SET_POSE, trackingUpdate);
		}
    	return false;
    }
    
    protected boolean wifiSendTuioExt(byte usbMsg, ACTOData ext) {
    	//TODO: check for ACTOData.type necessary?
    	if(wifiTransmitter != null)
		{
			return wifiTransmitter.sendUsbMsg(usbMsg, ext);
		}
    	return false;
    }




	
	//
	// COMMUNICATION TO USB ARDUINO
	//////////////////////
	
    //
    // COMMUNICATION INBOUND FROM USB HANDLING
    //////////////////
	public void handleUsbMsg(byte usb_msg, ACTOData data){
//		Log.d(TAG, "Received Data: " + usb_msg);
		
		//TODO: I do not have to know anything?
		tuioServer.onUsbMsg(usb_msg, data);
		
		//TODO: if I have to do something specific, address right method here, otherwise, just pipe through!
		switch (usb_msg) {

			// simple CMD messages			
			case (ACTOConst.USB_ACK):
				//do I want to know that here?
				//Log.d(TAG, "USB_ACK");
				break;
			
			case (ACTOConst.USB_PING):
//				Log.d(TAG, "TT PING: " + data.id);	//TODO: just debug, remove
//				Log.d(TAG, "RT STOP! ");
				break;
			
			//make no sense from usb??
//				case (ACTOConst.USB_GET_BYTE):
//					Log.d(TAG, "COMMAND: " + usb_msg + " for ID: " + data.id);	//TODO: just debug, remove
//					break;
//				
//				case (ACTOConst.USB_GET_CHAR):
//					Log.d(TAG, "COMMAND: " + usb_msg + " for ID: " + data.id);	//TODO: just debug, remove
//					break;
//				
//				case (ACTOConst.USB_GET_INT):
//					Log.d(TAG, "COMMAND: " + usb_msg + " for ID: " + data.id);	//TODO: just debug, remove
//					break;
//				
//				case (ACTOConst.USB_GET_FLOAT):
//					Log.d(TAG, "COMMAND: " + usb_msg + " for ID: " + data.id);	//TODO: just debug, remove
//					break;
			
			//case ACTOConst.USB_MOTOR: case ACTOConst.USB_MOTOR_MS: //makes no sense in that direction?
				
			case (ACTOConst.USB_SET_DI_UNIT):
				Log.d(TAG, "COMMAND: " + usb_msg + " for ID: " + data.id + ", data: " + data.iData);	//TODO: just debug, remove
				break;
			
			//case (ACTOConst.USB_SET_PWM_UNIT): //makes no sense in that direction?
			case (ACTOConst.USB_SET_AN_UNIT): 
				Log.d(TAG, "COMMAND: " + usb_msg + " for ID: " + data.id);	//TODO: just debug, remove
				break;
			
			case (ACTOConst.USB_SET_BYTE):
				Log.d(TAG, "COMMAND: " + usb_msg + " for ID: " + data.id + ", data: " + data.bData);	//TODO: just debug, remove
				break;
			
			case (ACTOConst.USB_SET_CHAR):
				Log.d(TAG, "COMMAND: " + usb_msg + " for ID: " + data.id + ", data: " + data.cData);	//TODO: just debug, remove
				//usbSerialComm.sendCmdIdDidChar(ACTOConst.USB_SET_CHAR, new ACTOData(ACTOData.DATA_CHAR, data.id, data.dataId, data.cData));//format: CMD, ID, dataID(byte), data(char16), SEP
				break;
			
			case (ACTOConst.USB_SET_INT):
				Log.d(TAG, "COMMAND: " + usb_msg + " for ID: " + data.id + ", data: " + data.iData);	//TODO: just debug, remove
				//usbSerialComm.sendCmdIdDidInt(ACTOConst.USB_SET_INT, new ACTOData(ACTOData.DATA_INT, data.id, data.dataId, data.iData));//format: CMD, ID, dataID(byte), data(int), SEP
				break;
			
			case (ACTOConst.USB_SET_FLOAT):
				Log.d(TAG, "COMMAND: " + usb_msg + " for ID: " + data.id + ", data: " + data.fData);	//TODO: just debug, remove
				//usbSerialComm.sendCmdIdDidFloat(ACTOConst.USB_SET_FLOAT, new ACTOData(ACTOData.DATA_FLOAT, data.id, data.dataId, data.fData));//format: CMD, ID, dataID(byte), data(float), SEP
				break;
				
			// message unknown					
			default:
				Log.e(TAG, "MESSAGE UNKNOWN: " + usb_msg);
				break;
		}
	}
		
	
    //
    // COMMUNICATION OUTBOUND OVER USB
    //////////////////
	protected boolean usbSendExt(byte usbMsg, ACTOData ad) {
    	//TODO: check for ACTOData.type necessary?
		if(usbSerialComm != null)
		{
			switch (usbMsg) {
		
				// simple CMD messages			
				case (ACTOConst.USB_ACK): case (ACTOConst.USB_NAK):
					return usbSerialComm.sendByte(usbMsg);
			
				case (ACTOConst.USB_PING): case (ACTOConst.USB_SYNC):
					return usbSerialComm.sendCmdId(usbMsg,ad);
			
				case (ACTOConst.USB_GET_BYTE): case (ACTOConst.USB_GET_CHAR):
				case (ACTOConst.USB_GET_INT): case (ACTOConst.USB_GET_FLOAT):
					return usbSerialComm.sendCmdIdDidEmpty(usbMsg,ad);
				
				case (ACTOConst.USB_MOTOR):
				case (ACTOConst.USB_SET_DI_UNIT): case (ACTOConst.USB_SET_BYTE):
					return usbSerialComm.sendCmdIdDidByte(usbMsg,ad);
				
				case (ACTOConst.USB_SET_CHAR):
					return usbSerialComm.sendCmdIdDidChar(usbMsg,ad);
				
				//case (ACTOConst.USB_SET_AN_UNIT):	//makes no sense in that direction?
				case (ACTOConst.USB_SET_PWM_UNIT): case (ACTOConst.USB_SET_INT):
					return usbSerialComm.sendCmdIdDidInt(usbMsg,ad);
		
				case (ACTOConst.USB_SET_FLOAT):
					return usbSerialComm.sendCmdIdDidFloat(usbMsg,ad);
				
				case (ACTOConst.USB_MOTOR_MS):
					return usbSerialComm.sendCmdIdDidByteInt(usbMsg,ad);
				
				// message unknown					
				default:
					Log.e(TAG, "MESSAGE UNKNOWN: " + usbMsg);
					return false;
			}
		}
		return false;
	}
		
		
	
	
	
	//
	// WIFI TRANSMITTER HANDLING
	//////////////

	/**
	 * Establish WiFi Transmitter
	 */
	private void establishWifiConnection(int port)
	{
		if(wifiTransmitter != null)
		{
			closeWifiConnection();
			Log.d(TAG, "WIFI WAS ESTABLISHED, SHUTDOWN");
		}
		
		Log.d(TAG, "ESTABLISH WIFI");
		wifiTransmitter = null;
		
		try 
		{
			wifiTransmitter = new WifiTransmitter(port, this); 
			wifiThread = new Thread(wifiTransmitter);
			wifiThread.start();
		} 
		catch (IOException e) 
		{
			Log.e(TAG, "unable to open Wifi Socket");
			Log.e(TAG, Log.getStackTraceString(e));
		}
	}

	/**
	 * Close WiFi Transmitter
	 */
	private void closeWifiConnection() 
	{
		if(wifiTransmitter != null)
		{
			
			wifiTransmitter.closeTransmitter();
			wifiTransmitter = null;
		}
		else
			Log.i(TAG, "wifitransmitter not instantiated");
	}
	
	//
	// WIFI TRANSMITTER HANDLING END
	//////////////
	
	//
	// USB SERIAL HANDLING
	//////////////

	/**
	 * Establish WiFi Transmitter
	 */
	private void establishUsbConnection(UsbManager usbManager)
	{
		if(usbSerialComm != null)
		{
			closeUsbConnection();
			Log.d(TAG, "WIFI WAS ESTABLISHED, SHUTDOWN");
		}
		
		Log.d(TAG, "ESTABLISH USB");
		usbSerialComm = null;
		
		try 
		{
			usbSerialComm = new UsbSerialComm(usbManager, this); 

			usbThread = new Thread(usbSerialComm);
			usbThread.start();
		} 
		catch (IOException e) 
		{
			Log.e(TAG, "unable to open USB Connection");
			Log.e(TAG, Log.getStackTraceString(e));
		}
	}

	/**
	 * Close WiFi Transmitter
	 */
	private void closeUsbConnection() 
	{
		if(usbSerialComm != null)
		{
			
			usbSerialComm.closeTransmitter();
			usbSerialComm = null;
		}
		else
			Log.i(TAG, "USB connection not instantiated");
	}
	
	//
	// USB SERIAL HANDLING END
	//////////////
	
    
    
    
       

}
