package at.ac.tuwien.ims.acto_framework;

import java.io.Closeable;
import java.io.IOException;
//import java.util.ArrayList;
//import java.util.Arrays;
import java.util.HashMap;
//import java.util.List;
import java.util.Map;

import org.apache.commons.math3.complex.Quaternion;
import org.apache.commons.math3.geometry.euclidean.threed.Rotation;
import org.apache.commons.math3.geometry.euclidean.threed.RotationOrder;
import org.apache.commons.math3.geometry.euclidean.threed.Vector3D;
import org.apache.commons.math3.geometry.euclidean.twod.Line;
import org.apache.commons.math3.geometry.euclidean.twod.Vector2D;

import android.util.Log;


/**
 * TUIO_Playground
 * 	handles pathplanning, playground conversions, etc 
 * 
 * @author Emanuel Vonach
 */
public class TUIOPlayground implements Closeable, Runnable {
	
	private final String TAG = this.getClass().getSimpleName();

	//status for TUIO positions
	private static final int PL_MARG_OUTSIDE = 0;
	private static final int PL_MARG_BORDER = 1;
	private static final int PL_MARG_INSIDE = 2;
	
	//default resolution
	public int[] trackingRes = {400, 300};		//TODO: calibrate outer boundary
	public int[] resolution = {800, 600, 200};	//Galaxy Nexus 720x1184
	
	public Vector3D calibVector = new Vector3D(0,0,400);
	public Quaternion calibRotation;
	
	private TUIOServer tuioServer;
	
    //Threadcontrol
    private boolean isLooping = true;
    //private boolean isCalib = false;
	
	//private List<TUIO> tuioList = new ArrayList<TUIO>();	//use List for pathpoints?
	
	// Populate the maps
	public Map<Byte,TUIO> markerMap = new HashMap<Byte,TUIO>();
	public Map<Byte,TUIO> tuioMap = new HashMap<Byte,TUIO>();
	
	//constructor
	public TUIOPlayground (TUIOServer callingServer)
	{
    	tuioServer = callingServer;
    	initializeTuios();	//asyncTask?
    	
    	Thread playThread = new Thread(this);
    	playThread.start();
    	Log.d(TAG, "started");
	}
	
	public boolean setSize(int[] resolution) {
		if (resolution.length == 3) {
			this.resolution = resolution;
			Log.d(TAG, "resolution set to: " + resolution[0] + " " + resolution[1] + " " + resolution[2]);
			return true;
		}
		Log.e(TAG, "playground resolution format invalid");
		return false;
	}
    
	public int[] getSize() {
		return resolution;
	}
	
	//TODO: only fast hack, proper loop required!!! 
	public boolean getAllPoseUpdate() {
		TUIO one = markerMap.get((byte)1);
		TUIO two = markerMap.get((byte)2);
		
		tuioServer.badTuioPoseHack(one.getPlayPose());
		tuioServer.badTuioPoseHack(two.getPlayPose());
		return true;
	}
	
	public ACTOData getTuioPose(byte id) {
		TUIO out = tuioMap.get(id);
		return out.getPlayPose();
	}

	public ACTOData getTuioTrans(byte id) {
		TUIO out = tuioMap.get(id);
		ACTOData outData = new ACTOData(ACTOData.TRANS, id, out.getPlayPose().translation);
		return outData;
	}

	public ACTOData getTuioRot(byte id) {
		
		TUIO out = tuioMap.get(id);
		ACTOData outData = new ACTOData(ACTOData.ROT, id, out.getPlayPose().rotation);
		return outData;
	}
	
	public void setTuioPose(ACTOData destPose) {
		//send to MotorControl, or TUIO.destination T, R
		TUIO currTUIO = tuioMap.get(destPose.id);
		
//		if (checkPlaygroundMargin(currTUIO) > PL_MARG_OUTSIDE) {
			currTUIO.driveTo(destPose);
//		}
//		else {
//			Log.e(TAG, "destination of ID '" + destPose.id + "' outside playground!");
//		}
		//TODO: check if already TUIO blocking place/path
	}
	
	public void setTuioTrans(ACTOData destPose) {
		//send to MotorControl, or TUIO.destination T + currentpose.R
		TUIO currTUIO = tuioMap.get(destPose.id);
		
//		if (checkPlaygroundMargin(currTUIO) > PL_MARG_OUTSIDE) {
			destPose.rotation = currTUIO.getPlayPose().rotation;
			currTUIO.driveTo(destPose);
//		}
//		else {
//			Log.e(TAG, "destination of ID " + (int)destPose.id + "outside playground margin!");
//		}
		//TODO: check if already TUIO blocking place/path
	}
	
	public void setTuioRot(ACTOData destPose) {
		//send to MotorControl, or TUIO.destination R + currentpose.T
		
		TUIO currTUIO = tuioMap.get(destPose.id);
		destPose.translation = currTUIO.getPlayPose().translation;
		currTUIO.driveTo(destPose);
	}
	
	
	
	
	//TODO IMPLEMENT
	
	private int checkPlaygroundMargin(TUIO currTUIO) {
		
//		Vector3D tuioVec3D = currTUIO.getTranslationVector();
//		//Galaxy Nexus 200x720x1184
//		//all borders have to be dependant on resolution
//		//what happens on lost tracking??
//		if (tuioVec3D.getX() <= 100 || tuioVec3D.getX() >= 900 || tuioVec3D.getY() <= 100 || tuioVec3D.getY() >= 800 || tuioVec3D.getZ() < -10 || tuioVec3D.getZ() >= 200)//resolution[0]) 
//			return PL_MARG_OUTSIDE;
//		else return PL_MARG_INSIDE;
		
		return PL_MARG_INSIDE;
		
	}

	
	
	
	
	
	
	
	private void setMotor(byte id, byte steerAngle, byte speed) {
		/* input int steeringAngle, int overallSpeed
		  steeringAngle 0-255: imagine as circle or steering wheel
		  0       = ++ straight ahead
		  1-31    = ++ slight right turn, right motor less than left one
		  32      = +0 right turn with right motor stop
		  33-63   = +- right turn with right motor help in counter direction
		  64      = +- right turn on spot, rightmotor = -leftmotor
		  65-95   = -+ right turn with right motor help in counter direction, but in reverse
		  96      = -0 right turn with right motor stop, but in reverse
		  97-127  = -- slight right turn, right motor less than left one, but in reverse
		  128     = -- straight reverse (with speed 0 active brake)
		  129-159 = -- slight left turn, left motor less than right one, but in reverse
		  160     = 0- left turn with left motor stop, but in reverse
		  161-191 = +- left turn with left motor help in counter direction, but in reverse
		  192     = -+ left turn on spot, leftmotor = -rightmotor
		  193-223 = -+ left turn with left motor help in counter direction
		  224     = 0+ left turn with left motor stop
		  225-255 = ++ slight left turn, left motor less than right one
		*/
		tuioServer.setTuioExt(ACTOConst.USB_MOTOR, new ACTOData(ACTOData.DATA_BYTE, id, steerAngle, speed));
	}
	
	private void setMotor(byte id, byte steerAngle, byte speed, int ms) {
		tuioServer.setTuioExt(ACTOConst.USB_MOTOR_MS, new ACTOData(ACTOData.MOTOR_MS, id, steerAngle, speed, ms));
	}
	
	public void startCalibration() {
		//isCalib = false;
		tuioServer.onCalibStateChange(ACTOConst.STATE_PENDING);
		
		//TODO: doCalibration(ACTOData calibPose)
	}
	
	public void resetCalibration() {
		//isCalib = false;
		tuioServer.onCalibStateChange(ACTOConst.STATE_CLOSED);
	}
	
	//TODO: implement properly
	//start calibration procedure
	public void doCalibration(ACTOData calibPose) {
		
		
		//TODO: WENN KEINE CALIB, DANN KEIN UPDATE MGLICH??
				//auf button klick check ob zwei marker sichtbar, wenn ja, dann speichere als playground limit & calib vector (mean?, beide?)
				
			
		
		
		
		//OLD:
		
		
		if (calibPose.translation[2] < 0)
		{
			//TUIO one = markerMap.get((byte)1);
			calibVector = new Vector3D((double)calibPose.translation[0], (double)calibPose.translation[1], (double)calibPose.translation[2]);
			calibRotation = new Quaternion((double)calibPose.rotation[3],(double)calibPose.rotation[0],(double)calibPose.rotation[1],(double)calibPose.rotation[2]);
			tuioServer.onCalibStateChange(ACTOConst.STATE_CONNECTED);
		}
	}
	

	private void initializeTuios() {
		//TODO: read from xml
		
    	//debug
    	TUIO test = new TUIO((byte)'1', (byte)1, this);
    	addTuio(test);
    	
    	TUIO test2 = new TUIO((byte)'2', (byte)2, this);
    	addTuio(test2);
    	
    	TUIO test3 = new TUIO((byte)'3', (byte)3, this);
    	addTuio(test3);
    	
    	TUIO test4 = new TUIO((byte)'4', (byte)4, this);
    	addTuio(test4);
	}
	
	
	public void addTuio(TUIO newTuio) {
	
		markerMap.put(newTuio.cosId, newTuio);
		tuioMap.put(newTuio.id, newTuio);
	}

	
	private void updateTUIO(ACTOData worldPose) {

		TUIO currTUIO = markerMap.get(worldPose.id);
		currTUIO.updateByWorldPose(worldPose);	
	}

	//incoming new tracking data, called every frame of tracker
	public void onNewTrackingData(ACTOData pose) {
		
		if (tuioServer.getState(TUIOServer.CALIB_VALID) == ACTOConst.STATE_CONNECTED) {

		
		
		
		
		
		
		
		//TODO: WENN KEINE CALIB, DANN KEIN UPDATE MGLICH??
		//auf button klick check ob zwei marker sichtbar, wenn ja, dann speichere als playground limit & calib vector (mean?, beide?)
		
		
		
		
		
		
			//Log.d(TAG, "CosID: " + pose.id + ", translation: " + pose.translation[0] + ", " + pose.translation[1] + ", " + pose.translation[2]);	//TODO: is changed by updateTUIO!!
		
			//update corresponding TUIO
			updateTUIO(pose);
	
			TUIO out = markerMap.get(pose.id);
			
			tuioServer.broadcastTuioPose(out.getPlayPose());
	
			//Vector3D bla = new Vector3D(out.playPose.translation[0],out.playPose.translation[1], out.playPose.translation[2]);
			
			//DEBUG
			//Log.d(TAG, "VECTOR Data: translation: " + out.getTranslationVector().getX() + ", " + out.getTranslationVector().getY() + ", " + out.getTranslationVector().getZ());
		}
		else
		{
			if (tuioServer.getState(TUIOServer.CALIB_VALID) == ACTOConst.STATE_PENDING) doCalibration(pose);
		}
			
	}
	
	public void testSth () {
//		if (isLooping)		isLooping = false;
//		else isLooping = true;
		
//		setMotor((byte)'1', (byte)64, (byte)5, 100);
		
		TUIO one = markerMap.get((byte)1);
		TUIO two = tuioMap.get((byte)'2');
		one.driveTo(two.getPlayPose());
		two.driveTo(one.getPlayPose());
	}

	@Override
	public void run() {
		while(isLooping)
		{
			drive();
			
			try {
				Thread.sleep(50);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	
	public void drive() {
		
		//variables to play around
		float toleranceOut = 1.2f;
		float toleranceIn = 0.1f;
		float toleranceRoute = 0.4f;
		//TODO: distance hardcoded, has to be relative to resolution!!
		int distanceGoal = 50;
		int distanceFastSlow = 140;
		
		byte straightSpeedFast = 20;
		byte straightSpeedSlow = 20;
		byte straightTime = 35;
		
		byte turnSpeedDirFast = 2;
		byte turnSpeedDirSlow = 5;	//actually faster, but stepwise
		int turnTimeDir = 35;
		byte turnSpeedRotFast = 2;
		byte turnSpeedRotSlow = 5;	//actually faster, but stepwise
		int turnTimeRot = 35;
		
		
		
		
		
		for (TUIO currTuio : tuioMap.values()) {
		
//			TUIO currTuio = tuioMap.get((byte)'1');
			if (currTuio.isDriving) {
			
				int marginPos = checkPlaygroundMargin(currTuio);
				
				if (marginPos != PL_MARG_OUTSIDE) {
					Vector2D curr2d = new Vector2D (currTuio.getTranslationVector().getX(), currTuio.getTranslationVector().getY());
					Vector2D dest2d = new Vector2D (currTuio.getDestinationVector().getX(), currTuio.getDestinationVector().getY());
					
	//				Log.d(TAG, "NEW TRK VAL (DR_play): " + curr2d.getX());		
					
					//for Cardan angles, the chosen set is the one for which the second angle is between -pi/2 and pi/2 (i.e its cosine is positive),
					Rotation currRot = new Rotation(currTuio.getRotationQuaternion().getQ1(),currTuio.getRotationQuaternion().getQ2(),currTuio.getRotationQuaternion().getQ3(),currTuio.getRotationQuaternion().getQ0(), true);
					double currAngle = currRot.getAngles(RotationOrder.XYZ)[2];//yaw, pitch and roll angle
					
		//			Log.d(TAG, "ANG RAD:" + //" X:" + (float)curr[0] + " Y:" + (float)curr[1] + 
		//			" Z:" + (float)curr[2]);
		//			Log.d(TAG, "ANG Grad: X:" + (float)(curr[0]*180/3.7603) + " Y:" + (float)(curr[1]*180/3.7603) + " Z:" + (float)(curr[2]*180/3.7603));
					
					//if target position not reached
					if (dest2d.distance(curr2d) > distanceGoal) {
						
						//calculate direction to target
						Line directionLine = new Line(curr2d, dest2d);
						double directionAngle = directionLine.getAngle();
						
						directionAngle = directionAngle-(Math.PI/2);	//get angle to yAxis
						
						//convert angle to comparable format
						if (directionAngle < 0){
							directionAngle = directionAngle+(Math.PI*2);
						}
						
						if (directionAngle > Math.PI) {
							directionAngle = directionAngle-(Math.PI*2);
						}
						
						directionAngle = directionAngle*-1; //invert
		
						//calculate angular difference
						float diff = (float)(directionAngle - currAngle);
						
						//check if turn over 180, change difference accordingly
						if (diff > Math.PI) {
							diff = diff - (float) Math.PI *2;
						}
						else if (diff < (Math.PI *-1)) {
							diff = diff + (float) Math.PI *2;
						}
						
	
						//if direction is either within inner tolerance, or already onRoute & within outer tolerance, drive straight
						//if (Math.abs(diff) > toleranceOut || (Math.abs(diff) > toleranceRoute && !currTuio.isOnRoute)) {
						if (Math.abs(diff) < toleranceIn || (Math.abs(diff) < toleranceRoute && currTuio.isOnRoute))
						{
	//							if (!currTuio.isOnRoute) setMotor(currTuio.id, (byte)128, (byte)0);
	//							else
	//							setMotor(currTuio.id, (byte)0, (byte)0);
							
							currTuio.isOnRoute = true;	
							
							//if closing towards target or near border of playground, drive slowly
							if (dest2d.distance(curr2d) < distanceFastSlow || marginPos == PL_MARG_BORDER) {
								setMotor(currTuio.id, (byte)0, straightSpeedSlow, straightTime);
								
	//							Log.d(TAG, "DIR STRAIGHT SLOW: " + diff);
	//							Log.d(TAG, "TT (PL_ST) after MOTOR: " + curr2d.getX() + "diff:" + diff);	
							}
							else {
								setMotor(currTuio.id, (byte)0, straightSpeedFast);
								
	//							Log.d(TAG, "DIR STRAIGHT FAST: " + diff);
	//							Log.d(TAG, "TT (PL_ST) after MOTOR: " + curr2d.getX() + "diff:" + diff);	
							}
						}
						//if not onRoute yet or going too far off direction, turn towards target
						else {
							
							currTuio.isOnRoute = false;
							
							if (Math.abs(diff) > toleranceOut) {
							
							
								if (diff > 0) {
	//								Log.d(TAG, "DIR RIGHT (OUTER TOL) (onScreen) " + diff);
									setMotor(currTuio.id, (byte)64, turnSpeedDirFast);
								}
								else
								{
	//								Log.d(TAG, "DIR LEFT (OUTER TOL) (onScreen) " + diff);
									setMotor(currTuio.id, (byte)192, turnSpeedDirFast);
								}
		//						Log.d(TAG, "UNGLEICH: " + diff);
		//						Log.d(TAG, "TT (PL_UN) after MOTOR: " + curr2d.getX()+ "diff:" + diff);		
							}
							else {
							
	//							if (Math.abs(diff) > toleranceRoute) {
	
								if (diff > 0) {
	//								Log.d(TAG, "DIR RIGHT (INNER TOL) (onScreen) " + diff);
									setMotor(currTuio.id, (byte)64, (byte)turnSpeedDirSlow, turnTimeDir);
								}
								else
								{
	//								Log.d(TAG, "DIR LEFT (INNER TOL) (onScreen) " + diff);
									setMotor(currTuio.id, (byte)192, (byte)turnSpeedDirSlow, turnTimeDir);
								}
	//							
	//							}
	
	//							else
	//								if (diff > 0) {
	//									Log.d(TAG, "DIR RIGHT (INNER TOL) (onScreen) " + diff);
	//									setMotor(currTuio.id, (byte)64, (byte)2, 30);
	//								}
	//								else
	//								{
	//									Log.d(TAG, "DIR LEFT (INNER TOL) (onScreen) " + diff);
	//									setMotor(currTuio.id, (byte)192, (byte)2, 30);
	//								}
	//							
							}
						}	
								
								
	
					}
		
					//if already on target position, match rotation
					else {
						Rotation destRot = new Rotation(currTuio.getDestinationQuaternion().getQ1(),currTuio.getDestinationQuaternion().getQ2(),currTuio.getDestinationQuaternion().getQ3(),currTuio.getDestinationQuaternion().getQ0(), true);
						double destAngle = destRot.getAngles(RotationOrder.XYZ)[2];//yaw, pitch and roll angle
		
		//				Log.d(TAG, "ANG2 RAD:" + //" X:" + (float)dest[0] + " Y:" + (float)dest[1] +
		//				" Z:" + (float)dest[2]);
						//Log.d(TAG, "ANG2 Grad: X:" + (float)(dest[0]*180/3.7603) + " Y:" + (float)(dest[1]*180/3.7603) + " Z:" + (float)(dest[2]*180/3.7603));
		
						
						//calculate angular difference
	//					float tol = 0.2f;	//const
						float diff = (float)(destAngle - currAngle);
						
						//check if turn over 180, change distance accordingly
						if (diff > Math.PI) {
							diff = diff - (float) Math.PI *2;
						}
						else if (diff < Math.PI *-1) {
							diff = diff + (float) Math.PI *2;
						}
						
						
						// if far too turn, turn fast
						if (Math.abs(diff) > toleranceOut) {
							
							if (diff > 0) {
	//							Log.d(TAG, "ROT RIGHT (OUTER TOL) (onScreen) " + diff);
								setMotor(currTuio.id, (byte)64, turnSpeedRotFast);
							}
							else
							{
	//							Log.d(TAG, "ROT LEFT (OUTER TOL) (onScreen) " + diff);
								setMotor(currTuio.id, (byte)192, turnSpeedRotFast);
							}
	//						Log.d(TAG, "UNGLEICH: " + diff);
	//						Log.d(TAG, "TT (PL_UN) after MOTOR: " + curr2d.getX()+ "diff:" + diff);		
						}
						//if already close to final orientation, turn slowly
						else if (Math.abs(diff) > toleranceIn) {
						
	//					if (Math.abs(diff) > toleranceRoute) {
	
							if (diff > 0) {
	//							Log.d(TAG, "ROT RIGHT (INNER TOL) (onScreen) " + diff);
								setMotor(currTuio.id, (byte)64, turnSpeedRotSlow, turnTimeRot);
							}
							else
							{
	//							Log.d(TAG, "ROT LEFT (INNER TOL) (onScreen) " + diff);
								setMotor(currTuio.id, (byte)192, turnSpeedRotSlow, turnTimeRot);
							}
	//					
						}
						//stop if target reached
						else{
	//						Log.d(TAG, "TARGET REACHED, STOP");
							setMotor(currTuio.id, (byte)0, (byte)0);
							currTuio.isDriving = false;
							currTuio.isOnRoute = false;
						}
					}
				}
				else {
					Log.e(TAG, "TUIO ID '" + currTuio.id + "' outside playground!");
					setMotor(currTuio.id, (byte)0, (byte)0);
					currTuio.isDriving = false;
					currTuio.isOnRoute = false;
				}
			}
		}
		
		//TODO: collision removed! keep all lines of TUIOs, if collision, handle (update lines always with new pos)
		
	}
	
	
	
//	public void directionMatch(TUIO one, TUIO two) {
//		
////		Vector3D dirVec = two.getTranslationVector().subtract(one.getTranslationVector());
////		Rotation rotOne = new Rotation(one.getRotationQuaternion().getQ1(),one.getRotationQuaternion().getQ2(),one.getRotationQuaternion().getQ3(),one.getRotationQuaternion().getQ0(), true);
////		
////		//Log.d(TAG, "VECTOR: X:" + two.getTranslationVector().getX() + " Y: " + two.getTranslationVector().getY() +" Z: " + two.getTranslationVector().getZ());
////		
////		try {
////			double ang = Vector3D.angle(Vector3D.PLUS_I, dirVec);
////
////			//Log.d(TAG, "ANGLE TUIO (to y): " + rotOne.getAngles(RotationOrder.XYZ)[2]);
////			
////			Log.d(TAG, "ANGLE TO TUIO (around Z): " + ang);
////			
//////			Rotation ya = new Rotation(Vector3D.PLUS_K, ang);
//////			
//////			Log.d(TAG, "ANGLE TO TARGET: " + ya.getAngles(RotationOrder.XYZ)[2]);
////		}
////		catch (Exception e) {
////			e.printStackTrace();
////		}
//
//		
//		
//		Rotation rotOne = new Rotation(one.getRotationQuaternion().getQ1(),one.getRotationQuaternion().getQ2(),one.getRotationQuaternion().getQ3(),one.getRotationQuaternion().getQ0(), true);
//
//		double zAngle = rotOne.getAngles(RotationOrder.XYZ)[2];
//				
//		Vector2D one2d = new Vector2D (one.getTranslationVector().getX(), one.getTranslationVector().getY());
//		Vector2D two2d = new Vector2D (two.getTranslationVector().getX(), two.getTranslationVector().getY());
//		
//		//Vector2D dirVec = two2d.subtract(one2d);
//		
//		//Vector3D dirVec3D = two.getTranslationVector().subtract(one.getTranslationVector());
//		//Vector2D dirVec = new Vector2D(dirVec3D.getX(), dirVec3D.getY());
//		
//		//Line dirLine = new Line(Vector2D.ZERO, dirVec);
//		Line dirLine = new Line(one2d, two2d);
//		
////		Log.d(TAG, "CURR VECTOR: X:" + one.getTranslationVector().getX() + " Y: " + one.getTranslationVector().getY());
////		Log.d(TAG, "DEST VECTOR: X:" + two.getTranslationVector().getX() + " Y: " + two.getTranslationVector().getY());
//		
////		Log.d(TAG, "CURR VECTOR: X:" + one2d.getX() + " Y: " + one2d.getY());
////		Log.d(TAG, "DEST VECTOR: X:" + two2d.getX() + " Y: " + two2d.getY());
////		Log.d(TAG, "ONE-TWO VECTOR: X:" + dirVec.getX() + " Y: " + dirVec.getY());
//		
//		
////		Log.d(TAG, "CURR ANGLE: Z:" + zAngle);
//		
//		double angLine = dirLine.getAngle();
////		Log.d(TAG, "ONE-TWO ANGLE ORIG: " + angLine);
//		
//		angLine = angLine-(Math.PI/2);	//get angle to yAxis
//		
//		if (angLine < 0){
//			angLine = angLine+(Math.PI*2);
//		}
//		
//		if (angLine > Math.PI) {
//			angLine = angLine-(Math.PI*2);
//		}
//		
//		angLine = angLine*-1; //invert
//		
////		Log.d(TAG, "ONE-TWO ANGLE CONV: " + angLine);
//		
//		
//		
//////	public void matchRotation (double currAngle, double destAngle, float tolerance) {
//
//		float tol = 0.4f;	//const
//		float diff = (float)(angLine - zAngle);
//		
//		//check if turn over 180, change distance accordingly
//		if (diff > Math.PI) {
//			diff = diff - (float) Math.PI *2;
//		}
//		else if (diff < (Math.PI *-1)) {
//			diff = diff + (float) Math.PI *2;
//		}
//		
//		//if (((float)curr[2] > (float)dest[2]+tol) || ((float)curr[2] < (float)dest[2]-tol))
//		if (Math.abs(diff) > tolerance)
//		{
////			Log.d(TAG, "UNGLEICH: " + diff);
////			float diff = (float)(curr[2] - dest[2]);
////			int direction = 192;
//					
//			if (diff > 0) {
////				Log.d(TAG, "nach rechts (aufScreen)");
//				setMotor((byte)'1', (byte)64, (byte)5);
//			}
//			else
//			{
////				Log.d(TAG, "nach links (aufScreen)");
//				setMotor((byte)'1', (byte)192, (byte)5);
//			}
//			
//		}
//		
//		else {
////			Log.d(TAG, "GLEICH");
////			
////			Log.d(TAG, "ONE-TWO distance: " + two2d.distance(one2d));
//					
////			if (two2d.distance(one2d) >= 170) {
////				setMotor((byte)'1', (byte)0, (byte)20);
////			}
////			else {
//			
//				setMotor((byte)'1', (byte)0, (byte)0);
////			}
//		}
//	}
	
	
	
	
	
	
	
	
//	
//	public void rotationMatch(TUIO one, TUIO two) {
//		Rotation rotOne = new Rotation(one.getRotationQuaternion().getQ1(),one.getRotationQuaternion().getQ2(),one.getRotationQuaternion().getQ3(),one.getRotationQuaternion().getQ0(), true);
//		Rotation rotTwo = new Rotation(two.getRotationQuaternion().getQ1(),two.getRotationQuaternion().getQ2(),two.getRotationQuaternion().getQ3(),two.getRotationQuaternion().getQ0(), true);
//
//		//for Cardan angles, the chosen set is the one for which the second angle is between -pi/2 and pi/2 (i.e its cosine is positive),
//		double curr[] = rotOne.getAngles(RotationOrder.XYZ);	//yaw, pitch and roll angle
////		Log.d(TAG, "ANG RAD:" + //" X:" + (float)curr[0] + " Y:" + (float)curr[1] + 
////		" Z:" + (float)curr[2]);
////		Log.d(TAG, "ANG Grad: X:" + (float)(curr[0]*180/3.7603) + " Y:" + (float)(curr[1]*180/3.7603) + " Z:" + (float)(curr[2]*180/3.7603));
//		
//		double dest[] = rotTwo.getAngles(RotationOrder.XYZ);
////		Log.d(TAG, "ANG2 RAD:" + //" X:" + (float)dest[0] + " Y:" + (float)dest[1] +
////		" Z:" + (float)dest[2]);
//		//Log.d(TAG, "ANG2 Grad: X:" + (float)(dest[0]*180/3.7603) + " Y:" + (float)(dest[1]*180/3.7603) + " Z:" + (float)(dest[2]*180/3.7603));
//
//		float tol = 0.4f;	//const
//		float diff = (float)(dest[2] - curr[2]);
//		
//		//check if turn over 180, change distance accordingly
//		if (diff > Math.PI) {
//			diff = diff - (float) Math.PI *2;
//		}
//		else if (diff < Math.PI *-1) {
//			diff = diff + (float) Math.PI *2;
//		}
//		
//		//if (((float)curr[2] > (float)dest[2]+tol) || ((float)curr[2] < (float)dest[2]-tol))
//		if (Math.abs(diff) > tol)
//		{
//			Log.d(TAG, "UNGLEICH: " + diff);
////			float diff = (float)(curr[2] - dest[2]);
////			int direction = 192;
//			
//			if (diff > 0) {
////				Log.d(TAG, "nach rechts (aufScreen)");
//				setMotor((byte)'1', (byte)64, (byte)5);
//			}
//			else
//			{
////				Log.d(TAG, "nach links (aufScreen)");
//				setMotor((byte)'1', (byte)192, (byte)5);
//			}
//			
//		}
//		else{
//			Log.d(TAG, "GLEICH");
//			setMotor((byte)'1', (byte)0, (byte)0);
//		}
//
//		
//	}
	
	
	
	
	
	@Override
	public void close() throws IOException {
		// TODO: close all threads etc
		setMotor((byte)'1', (byte)0, (byte)0);
		isLooping = false;
		try {
			Thread.sleep(100);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Log.d(TAG, "playground cleaned up");
	}



	





	

	
	
	
	
//protected void moveListener() {
//	//ID, T, R
//}
	
	
	    
   
    



}
